# Generated by Django 2.2 on 2019-09-20 20:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("app_add_manytomany_field", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="b",
            name="many_to_many",
            field=models.ManyToManyField(to="app_add_manytomany_field.A"),
        )
    ]
