# Generated by Django 2.1.10 on 2019-07-19 09:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("app_drop_table", "0001_initial")]

    operations = [migrations.DeleteModel(name="A")]
