# Generated by Django 2.2 on 2019-07-29 21:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("app_unique_together", "0002_auto_20190729_2122")]

    operations = [migrations.AlterUniqueTogether(name="a", unique_together=set())]
