# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-04-14 15:02
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("app_rename_column", "0001_initial")]

    operations = [
        migrations.RenameField(model_name="a", old_name="field", new_name="renamed")
    ]
