# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import AppConfig


class AppCreateTableWithNotNullColumnConfig(AppConfig):
    name = "tests.test_project.app_create_table_with_not_null_column"
